@@ -9,7 +9,7 @@ use HTTP::Request;
 use File::Spec;
 use vars qw($VERSION);
 
-$VERSION = '1.40';
+$VERSION = '1.42';
 
 sub recent {
   my $package = shift;
@@ -21,12 +21,12 @@ sub recent {
   my $options = delete $opts{options};
   my $self = bless \%opts, $package;
   $self->{uri} = URI->new( $self->{url} );
-  croak "url provided is of an unsupported scheme\n" 
+  croak "url provided is of an unsupported scheme\n"
 	unless $self->{uri}->scheme and $self->{uri}->scheme =~ /^(ht|f)tp|file$/;
   $self->{session_id} = POE::Session->create(
 	object_states => [
 	   $self => [ qw(_start _process_http _process_ftp _process_file _recent _sig_child _epoch _epoch_fail) ],
-	   $self => { 
+	   $self => {
 		      http_sockerr  => '_get_connect_error',
 		      http_timeout  => '_get_connect_error',
 		      http_response => '_http_response',
@@ -151,6 +151,7 @@ sub _get_error {
 
 sub _get_data {
   my ($kernel,$self,$data) = @_[KERNEL,OBJECT,ARG0];
+  $data =~ s![\x0D\x0A]+$!!g;
   if ( $self->{rss} ) {
     return unless $data =~ m#<link>(.+?)</link>#i;
     return unless $data =~ m#by-authors#i;
@@ -179,8 +180,12 @@ sub _get_done {
 sub _process_file {
   my ($kernel,$self) = @_[KERNEL,OBJECT];
   delete $self->{_epoch_fail};
-  my $path = File::Spec->rel2abs( $self->{uri}->path );
-  $path = File::Spec->catfile( $path, 'RECENT' );
+  {
+    my @segs = $self->{uri}->path_segments;
+    pop @segs unless $segs[-1];
+    push @segs, 'RECENT';
+    $self->{uri}->path_segments( @segs );
+  }
   $self->{wheel} = POE::Wheel::Run->new(
       Program => sub {
         my $path = shift;
@@ -190,7 +195,7 @@ sub _process_file {
         }
         close $fh;
       },
-      ProgramArgs => [ $path ],
+      ProgramArgs => [ $self->{uri}->file ],
       StdoutEvent => 'ftp_data',
   );
   $kernel->sig_child( $self->{wheel}->PID(), '_sig_child' );
@@ -204,7 +209,7 @@ sub _epoch {
       Program => sub {
         my $epoch  = shift;
         my $mirror = shift;
-        print STDOUT $_, "\n" for 
+        print STDOUT $_, "\n" for
           CPAN::Recent::Uploads->recent( $epoch, $mirror );
       },
       ProgramArgs => [ $self->{epoch}, $self->{uri}->as_string ],
@@ -255,7 +260,7 @@ POE::Component::SmokeBox::Recent - A POE component to retrieve recent CPAN uploa
   exit 0;
 
   sub _start {
-    POE::Component::SmokeBox::Recent->recent( 
+    POE::Component::SmokeBox::Recent->recent(
 	url => 'http://www.cpan.org/',
 	event => 'recent',
     );
@@ -274,7 +279,7 @@ POE::Component::SmokeBox::Recent - A POE component to retrieve recent CPAN uploa
 
 =head1 DESCRIPTION
 
-POE::Component::SmokeBox::Recent is a L<POE> component for retrieving recently uploaded CPAN distributions 
+POE::Component::SmokeBox::Recent is a L<POE> component for retrieving recently uploaded CPAN distributions
 from the CPAN mirror of your choice.
 
 It accepts a url and an event name and attempts to download and parse the RECENT file from that given url.
@@ -299,13 +304,13 @@ Takes a number of parameters:
 The 'session' parameter is only required if you wish the output event to go to a different
 session than the calling session, or if you have spawned the poco outside of a session.
 
-The 'rss' parameter if set will indicate that the poco should retrieve recent uploads from the 
-C<modules/01modules.mtime.rss> file instead of the C<RECENT> file. The rss file contains the 
+The 'rss' parameter if set will indicate that the poco should retrieve recent uploads from the
+C<modules/01modules.mtime.rss> file instead of the C<RECENT> file. The rss file contains the
 150 most recent uploads to CPAN and is more up to date than the C<RECENT> file.
 
 The 'epoch' parameter should be a valid epoch timestamp less than the current time but greater than
 a year ago. Setting this will cause the component to use L<CPAN::Recent::Uploads> to obtain a list
-of distributions uploaded since the 'epoch' time given. This enables more grandular control of 
+of distributions uploaded since the 'epoch' time given. This enables more grandular control of
 listing dists than simply retrieving the C<RECENT> file.
 
 The poco does it's work and will return the output event with the result.
@@ -316,7 +321,7 @@ The poco does it's work and will return the output event with the result.
 
 This is generated by the poco. ARG0 will be a hash reference with the following keys:
 
-  'recent', an arrayref containing recently uploaded distributions; 
+  'recent', an arrayref containing recently uploaded distributions;
   'error', if something went wrong this will contain some hopefully meaningful error messages;
   'context', if you supplied a context in the constructor it will be returned here;
 
@@ -332,7 +337,7 @@ This module may be used, modified, and distributed under the same terms as Perl
 
 =head1 KUDOS
 
-Andy Armstrong for helping me to debug accessing his CPAN mirror. 
+Andy Armstrong for helping me to debug accessing his CPAN mirror.
 
 =head1 SEE ALSO
 
